<?php
/**
 * FilterModel class
 *
 * PHP version 5
 *
 * @category Packages
 * @package  Ext.model
 * @author   Evgeniy Marilev <jeka@5-soft.com>
 * @license  http://5-soft.com/packages/php/license proprietary
 * @link     http://5-soft.com/packages/php/docs
 */
/**
 * FilterModel is the base filter model class used for filtering data by html form
 * 
 * @category Packages
 * @package  Ext.model
 * @author   Evgeniy Marilev <jeka@5-soft.com>
 * @license  http://5-soft.com/packages/php/license proprietary
 * @link     http://5-soft.com/packages/php/docs
 */
class FilterModel extends CFormModel implements IFilterModel
{
    const SCENARIO_FILTERING = 'filtering';
    
    /**
     * Constructor.
     * 
     * @param string $scenario name of the scenario that this model is used in.
     * 
     * @return void
     */
    public function __construct($scenario = self::SCENARIO_FILTERING)
    {
        parent::__construct($scenario);
    }
    
    /**
     * Applies filtering to existing criteria
     * 
     * @param CDbCriteria $criteria database criteria
     * 
     * @return void
     * @see IFilterModel::apply();
     */
    public function apply($criteria)
    {
        if ($this->validate()) {
            foreach ($this->getSafeAttributeNames() as $attribute) {
                $this->applyAttributeFilter($criteria, $attribute);
            }
        } else {
            $errors = $this->getErrors();
            $this->displayErrors($errors);
        }
    }
    
    /**
     * Applies filtering to existing criteria only for special filter's attribute
     * 
     * @param CDbCriteria $criteria  database criteria
     * @param string      $attribute attribute name
     * 
     * @return void
     */
    public function applyAttributeFilter($criteria, $attribute)
    {
        $criteria->compare('t.' . $attribute, $this->$attribute);
    }
    
    /**
     * Displays validation errors
     * 
     * @param array &$data errors map
     * 
     * @return void
     * @throws Exception
     */
    protected function displayErrors(&$data)
    {
        $errors = array();
        foreach (array_values($data) as $error) {
            $pieces = array_merge($errors, $error);
        }
        $params = array('{ERRORS}' => implode(', ', $errors));
        $message = 'Filter attributes validation errors: {ERRORS}.';
        $message = Yii::t('app', $message, $params);
        throw new Exception($message);
    }
}