<?php
/**
 * Sandbox revision behavior.
 *
 * Should be attached to the revision model:
 *    public function behaviors() {
 *        return array(
 *            'RevisionBehavior' => array(
 *                'class' => 'application.models.sandbox.RevisionBehavior',
 *                'sandboxField' => 'publisher_profile_id',
 *            ),
 *        );
 *    }
 * sandboxField parameter is required - field name where main sandbox model id is stored
 *
 * @author seb.goo
 * @package app.models
 * @subpackage app.models.sandbox
 */
class RevisionBehavior extends CActiveRecordBehavior {
      /**
       * sanbox revision
       * @var integer
       */
      const SANDBOX = 0;
      /**
       * history revision
       * @var integer
       */
      const HISTORY = 1;
      /**
       * published revision
       * @var integer
       */
      const PUBLISHED = 2;

      /**
       * The name of main sandboc model class.
       * @var string
       */
      public $sandboxModelClass;

      /**
       * Field name where main sandbox model id is stored
       * @var string
       */
      public $sandboxField;
      /**
       * Field name where revision type is stored
       * @var string
       */
      public $revisionTypeField = 'revision_type';

      /**
       * Field name where revision name is stored
       * @var string
       */
      public $revisionNameField = 'revision_name';

      /**
       * Field name where revision name is stored
       * @var string
       */
      public $revisionCreatorField = 'creator_id';

      /**
       * The name of the relation which will be added to the owner model.
       * @var string
       */
      public $sandboxRelationName = 'sandbox';

      /**
       *
       * @var string revision label
       */
      public $revisionLabel = 'Revision';

      /**
       * Get field name where main sandbox model id is stored
       */
      public function getSandboxField() {
          return $this->sandboxField;
      }

      /**
       * Get field name where revision type is stored
       */
      public function getRevisionTypeField() {
          return $this->revisionTypeField;
      }

      /**
       *
       * @return string revision label
       */
      public function getRevisionLabel() {
          return $this->revisionLabel;
      }
      
      /**
       * Get main sandbox model id
       */
      public function getSandboxId() {
          return $this->owner->{$this->sandboxField};
      }
      
     /**
       * Get main sandbox model
       */
      public function getSandbox() {
          return $this->owner->{$this->sandboxRelationName};
      }

      /**
       * Set main sandbox model id
       * @param $value
       */
      public function setSandboxId($value) {
          $this->owner->{$this->sandboxField} = $value;
      }

      /**
       * Get revision type
       */
      public function getRevisionType() {
          return $this->owner->{$this->revisionTypeField};
      }

      /**
       * Set revision type
       * @param $value
       */
      public function setRevisionType($value) {
          $this->owner->{$this->revisionTypeField} = $value;
      }

      /**
       * Get revision type
       */
      public function getRevisionName() {
          return $this->owner->{$this->revisionNameField};
      }

      /**
       * Set revision type
       * @param $value
       */
      public function setRevisionName($value) {
          $this->owner->{$this->revisionNameField} = $value;
      }

            /**
       * Get revision type
       */
      public function getCreatorId() {
          return $this->owner->{$this->revisionCreatorField};
      }

      /**
       * Set revision type
       * @param $value
       */
      public function setCreatorId($value) {
          $this->owner->{$this->revisionCreatorField} = $value;
      }

    /**
     * Adds relation to the sandbox when attaching to the owner model
     */
    public function attach($owner) {
        parent::attach($owner);
        $owner->getMetaData()->addRelation($this->sandboxRelationName, array(
            CActiveRecord::BELONGS_TO, $this->sandboxModelClass, $this->sandboxField
        ));
    }

    /**
     * Overrides parent implementation to set owner's creator and revision name
     * @see CActiveRecordBehavior::beforeSave()
     */
    public function beforeSave($on)
    {
        if ($this->owner->isNewRecord) {
            $this->setCreatorId(People::loggedUser()->id);
            if ($this->getRevisionType() != SandboxBehavior::SANDBOX_REVISION) {
                $this->setRevisionName('Revision ' . ($this->getSandbox()->getCounter('revision')));
            }
        }
        return parent::afterSave($on);
    }

    /**
     * Overrides parent implementation to increase owner's revision counter
     * @see CActiveRecordBehavior::afterSave()
     */
    public function afterSave($event) {
        if ($this->owner->isNewRecord) {
            if ($this->getRevisionType() != SandboxBehavior::SANDBOX_REVISION) {
                $this->getSandbox()->incCounter('revision');
            }
        }
        return parent::afterSave($event);
    }
}
